@@ -22,10 +22,14 @@ import (
 	"github.com/mikesmitty/edkey"
 	"github.com/pires/go-proxyproto"
 	"github.com/stretchr/testify/require"
-	gitalyClient "gitlab.com/gitlab-org/gitaly/v15/client"
-	pb "gitlab.com/gitlab-org/gitaly/v15/proto/go/gitalypb"
-	"gitlab.com/gitlab-org/gitlab-shell/v14/internal/testhelper"
+	gitalyClient "gitlab.com/gitlab-org/gitaly/v16/client"
+	pb "gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
+	"gitlab.com/gitlab-org/gitaly/v16/streamio"
 	"golang.org/x/crypto/ssh"
+	"google.golang.org/grpc"
+
+	"gitlab.com/gitlab-org/gitlab-shell/v14/client/testserver"
+	"gitlab.com/gitlab-org/gitlab-shell/v14/internal/testhelper"
 )
 
 var (
@@ -67,7 +71,7 @@ func rootDir() string {
 	return filepath.Join(filepath.Dir(currentFile), "..", "..")
 }
 
-func ensureGitalyRepository(t *testing.T) {
+func ensureGitalyRepository(t *testing.T) (*grpc.ClientConn, *pb.Repository) {
 	if os.Getenv("GITALY_CONNECTION_INFO") == "" {
 		t.Skip("GITALY_CONNECTION_INFO is not set")
 	}
@@ -83,47 +87,106 @@ func ensureGitalyRepository(t *testing.T) {
 	_, err = namespace.RemoveNamespace(context.Background(), rmNsReq)
 	require.NoError(t, err)
 
-	gl_repository := &pb.Repository{StorageName: gitalyConnInfo.Storage, RelativePath: testRepo}
-	createReq := &pb.CreateRepositoryFromURLRequest{Repository: gl_repository, Url: testRepoImportUrl}
+	glRepository := &pb.Repository{StorageName: gitalyConnInfo.Storage, RelativePath: testRepo}
+	createReq := &pb.CreateRepositoryFromURLRequest{Repository: glRepository, Url: testRepoImportUrl}
 
 	_, err = repository.CreateRepositoryFromURL(context.Background(), createReq)
 	require.NoError(t, err)
+
+	return conn, glRepository
 }
 
-func successAPI(t *testing.T) http.Handler {
+func startGitOverHTTPServer(t *testing.T) string {
+	ctx := context.Background()
+	conn, glRepository := ensureGitalyRepository(t)
+	client := pb.NewSmartHTTPServiceClient(conn)
+
+	requests := []testserver.TestRequestHandler{
+		{
+			Path: "/info/refs",
+			Handler: func(w http.ResponseWriter, r *http.Request) {
+				rpcRequest := &pb.InfoRefsRequest{
+					Repository: glRepository,
+				}
+
+				var reader io.Reader
+				switch r.URL.Query().Get("service") {
+				case "git-receive-pack":
+					stream, err := client.InfoRefsReceivePack(ctx, rpcRequest)
+					require.NoError(t, err)
+					reader = streamio.NewReader(func() ([]byte, error) {
+						resp, err := stream.Recv()
+						return resp.GetData(), err
+					})
+				default:
+					t.FailNow()
+				}
+
+				_, err := io.Copy(w, reader)
+				require.NoError(t, err)
+			},
+		},
+		{
+			Path: "/git-receive-pack",
+			Handler: func(w http.ResponseWriter, r *http.Request) {
+				body, err := io.ReadAll(r.Body)
+				require.NoError(t, err)
+				defer r.Body.Close()
+
+				require.Equal(t, string(body), "0000")
+			},
+		},
+	}
+
+	return testserver.StartHttpServer(t, requests)
+}
+
+func buildAllowedResponse(t *testing.T, filename string) string {
+	testhelper.PrepareTestRootDir(t)
+
+	body, err := os.ReadFile(filepath.Join(testhelper.TestRoot, filename))
+	require.NoError(t, err)
+
+	response := strings.Replace(string(body), "GITALY_REPOSITORY", testRepo, 1)
+
+	if gitalyConnInfo != nil {
+		response = strings.Replace(response, "GITALY_ADDRESS", gitalyConnInfo.Address, 1)
+		response = strings.Replace(response, "GITALY_STORAGE", gitalyConnInfo.Storage, 1)
+	}
+
+	return response
+}
+
+type customHandler struct {
+	url    string
+	caller http.HandlerFunc
+}
+
+func successAPI(t *testing.T, handlers ...customHandler) http.Handler {
 	t.Helper()
 
 	return http.HandlerFunc(func(w http.ResponseWriter, r *http.Request) {
-		testhelper.PrepareTestRootDir(t)
-
 		t.Logf("gitlab-api-mock: received request: %s %s", r.Method, r.RequestURI)
 		w.Header().Set("Content-Type", "application/json")
 
-		switch r.URL.EscapedPath() {
+		url := r.URL.EscapedPath()
+
+		for _, handler := range handlers {
+			if url == handler.url {
+				handler.caller(w, r)
+
+				return
+			}
+		}
+
+		switch url {
 		case "/api/v4/internal/authorized_keys":
 			fmt.Fprintf(w, `{"id":1, "key":"%s"}`, r.FormValue("key"))
-		case "/api/v4/internal/discover":
-			fmt.Fprint(w, `{"id": 1000, "name": "Test User", "username": "test-user"}`)
-		case "/api/v4/internal/personal_access_token":
-			fmt.Fprint(w, `{"success": true, "token": "testtoken", "scopes": ["api"], "expires_at": ""}`)
-		case "/api/v4/internal/two_factor_recovery_codes":
-			fmt.Fprint(w, `{"success": true, "recovery_codes": ["code1", "code2"]}`)
-		case "/api/v4/internal/two_factor_otp_check":
-			fmt.Fprint(w, `{"success": true}`)
 		case "/api/v4/internal/allowed":
-			body, err := os.ReadFile(filepath.Join(testhelper.TestRoot, "responses/allowed_without_console_messages.json"))
+			response := buildAllowedResponse(t, "responses/allowed_without_console_messages.json")
+
+			_, err := fmt.Fprint(w, response)
 			require.NoError(t, err)
-
-			response := strings.Replace(string(body), "GITALY_REPOSITORY", testRepo, 1)
-
-			if gitalyConnInfo != nil {
-				response = strings.Replace(response, "GITALY_ADDRESS", gitalyConnInfo.Address, 1)
-			}
-
-			fmt.Fprint(w, response)
-			require.NoError(t, err)
-		case "/api/v4/internal/lfs_authenticate":
-			fmt.Fprint(w, `{"username": "test-user", "lfs_token": "testlfstoken", "repo_path": "foo", "expires_in": 7200}`)
 		default:
 			t.Logf("Unexpected request to successAPI: %s", r.URL.EscapedPath())
 			t.FailNow()
@@ -273,7 +336,13 @@ func runSSHD(t *testing.T, apiHandler http.Handler) *ssh.Client {
 }
 
 func TestDiscoverSuccess(t *testing.T) {
-	client := runSSHD(t, successAPI(t))
+	handler := customHandler{
+		url: "/api/v4/internal/discover",
+		caller: func(w http.ResponseWriter, _ *http.Request) {
+			fmt.Fprint(w, `{"id": 1000, "name": "Test User", "username": "test-user"}`)
+		},
+	}
+	client := runSSHD(t, successAPI(t, handler))
 
 	session, err := client.NewSession()
 	require.NoError(t, err)
@@ -285,7 +354,13 @@ func TestDiscoverSuccess(t *testing.T) {
 }
 
 func TestPersonalAccessTokenSuccess(t *testing.T) {
-	client := runSSHD(t, successAPI(t))
+	handler := customHandler{
+		url: "/api/v4/internal/personal_access_token",
+		caller: func(w http.ResponseWriter, _ *http.Request) {
+			fmt.Fprint(w, `{"success": true, "token": "testtoken", "scopes": ["api"], "expires_at": "9001-01-01"}`)
+		},
+	}
+	client := runSSHD(t, successAPI(t, handler))
 
 	session, err := client.NewSession()
 	require.NoError(t, err)
@@ -293,25 +368,22 @@ func TestPersonalAccessTokenSuccess(t *testing.T) {
 
 	output, err := session.Output("personal_access_token test api")
 	require.NoError(t, err)
-	require.Equal(t, "Token:   testtoken\nScopes:  api\nExpires: never\n", string(output))
+	require.Equal(t, "Token:   testtoken\nScopes:  api\nExpires: 9001-01-01\n", string(output))
 }
 
 func TestTwoFactorAuthRecoveryCodesSuccess(t *testing.T) {
-	client := runSSHD(t, successAPI(t))
-
-	session, err := client.NewSession()
-	require.NoError(t, err)
-	defer session.Close()
-
-	stdin, err := session.StdinPipe()
-	require.NoError(t, err)
-
-	stdout, err := session.StdoutPipe()
-	require.NoError(t, err)
+	handler := customHandler{
+		url: "/api/v4/internal/two_factor_recovery_codes",
+		caller: func(w http.ResponseWriter, _ *http.Request) {
+			fmt.Fprint(w, `{"success": true, "recovery_codes": ["code1", "code2"]}`)
+		},
+	}
+	client := runSSHD(t, successAPI(t, handler))
+	session, stdin, stdout := newSession(t, client)
 
 	reader := bufio.NewReader(stdout)
 
-	err = session.Start("2fa_recovery_codes")
+	err := session.Start("2fa_recovery_codes")
 	require.NoError(t, err)
 
 	line, err := reader.ReadString('\n')
@@ -340,21 +412,18 @@ a new device so you do not lose access to your account again.
 }
 
 func TwoFactorAuthVerifySuccess(t *testing.T) {
-	client := runSSHD(t, successAPI(t))
-
-	session, err := client.NewSession()
-	require.NoError(t, err)
-	defer session.Close()
-
-	stdin, err := session.StdinPipe()
-	require.NoError(t, err)
-
-	stdout, err := session.StdoutPipe()
-	require.NoError(t, err)
+	handler := customHandler{
+		url: "/api/v4/internal/two_factor_otp_check",
+		caller: func(w http.ResponseWriter, _ *http.Request) {
+			fmt.Fprint(w, `{"success": true}`)
+		},
+	}
+	client := runSSHD(t, successAPI(t, handler))
+	session, stdin, stdout := newSession(t, client)
 
 	reader := bufio.NewReader(stdout)
 
-	err = session.Start("2fa_verify")
+	err := session.Start("2fa_verify")
 	require.NoError(t, err)
 
 	line, err := reader.ReadString('\n')
@@ -370,7 +439,13 @@ func TwoFactorAuthVerifySuccess(t *testing.T) {
 }
 
 func TestGitLfsAuthenticateSuccess(t *testing.T) {
-	client := runSSHD(t, successAPI(t))
+	handler := customHandler{
+		url: "/api/v4/internal/lfs_authenticate",
+		caller: func(w http.ResponseWriter, _ *http.Request) {
+			fmt.Fprint(w, `{"username": "test-user", "lfs_token": "testlfstoken", "repo_path": "foo", "expires_in": 7200}`)
+		},
+	}
+	client := runSSHD(t, successAPI(t, handler))
 
 	session, err := client.NewSession()
 	require.NoError(t, err)
@@ -387,17 +462,46 @@ func TestGitReceivePackSuccess(t *testing.T) {
 	ensureGitalyRepository(t)
 
 	client := runSSHD(t, successAPI(t))
-	session, err := client.NewSession()
-	require.NoError(t, err)
-	defer session.Close()
+	session, stdin, stdout := newSession(t, client)
 
-	stdin, err := session.StdinPipe()
+	err := session.Start(fmt.Sprintf("git-receive-pack %s", testRepo))
 	require.NoError(t, err)
 
-	stdout, err := session.StdoutPipe()
+	// Gracefully close connection
+	_, err = fmt.Fprintln(stdin, "0000")
+	require.NoError(t, err)
+	stdin.Close()
+
+	output, err := io.ReadAll(stdout)
 	require.NoError(t, err)
 
-	err = session.Start(fmt.Sprintf("git-receive-pack %s", testRepo))
+	outputLines := strings.Split(string(output), "\n")
+
+	for i := 0; i < (len(outputLines) - 1); i++ {
+		require.Regexp(t, "^[0-9a-f]{44} refs/(heads|tags)/[^ ]+", outputLines[i])
+	}
+
+	require.Equal(t, "0000", outputLines[len(outputLines)-1])
+}
+
+func TestGeoGitReceivePackSuccess(t *testing.T) {
+	url := startGitOverHTTPServer(t)
+
+	handler := customHandler{
+		url: "/api/v4/internal/allowed",
+		caller: func(w http.ResponseWriter, _ *http.Request) {
+			response := buildAllowedResponse(t, "responses/allowed_with_geo_push_payload.json")
+			response = strings.Replace(response, "PRIMARY_REPO", url, 1)
+
+			w.WriteHeader(300)
+			_, err := fmt.Fprint(w, response)
+			require.NoError(t, err)
+		},
+	}
+	client := runSSHD(t, successAPI(t, handler))
+	session, stdin, stdout := newSession(t, client)
+
+	err := session.Start(fmt.Sprintf("git-receive-pack %s", testRepo))
 	require.NoError(t, err)
 
 	// Gracefully close connection
@@ -421,62 +525,51 @@ func TestGitUploadPackSuccess(t *testing.T) {
 	ensureGitalyRepository(t)
 
 	client := runSSHD(t, successAPI(t))
-	session, err := client.NewSession()
-	require.NoError(t, err)
-	defer session.Close()
+	defer client.Close()
 
-	stdin, err := session.StdinPipe()
-	require.NoError(t, err)
+	numberOfSessions := 3
+	for sessionNumber := 0; sessionNumber < numberOfSessions; sessionNumber++ {
+		t.Run(fmt.Sprintf("session #%v", sessionNumber), func(t *testing.T) {
+			session, stdin, stdout := newSession(t, client)
+			reader := bufio.NewReader(stdout)
 
-	stdout, err := session.StdoutPipe()
-	require.NoError(t, err)
+			err := session.Start(fmt.Sprintf("git-upload-pack %s", testRepo))
+			require.NoError(t, err)
 
-	reader := bufio.NewReader(stdout)
+			line, err := reader.ReadString('\n')
+			require.NoError(t, err)
+			require.Regexp(t, "^[0-9a-f]{44} HEAD.+", line)
 
-	err = session.Start(fmt.Sprintf("git-upload-pack %s", testRepo))
-	require.NoError(t, err)
+			// Gracefully close connection
+			_, err = fmt.Fprintln(stdin, "0000")
+			require.NoError(t, err)
 
-	line, err := reader.ReadString('\n')
-	require.NoError(t, err)
-	require.Regexp(t, "^[0-9a-f]{44} HEAD.+", line)
+			output, err := io.ReadAll(stdout)
+			require.NoError(t, err)
 
-	// Gracefully close connection
-	_, err = fmt.Fprintln(stdin, "0000")
-	require.NoError(t, err)
+			outputLines := strings.Split(string(output), "\n")
 
-	output, err := io.ReadAll(stdout)
-	require.NoError(t, err)
+			for i := 1; i < (len(outputLines) - 1); i++ {
+				require.Regexp(t, "^[0-9a-f]{44} refs/(heads|tags)/[^ ]+", outputLines[i])
+			}
 
-	outputLines := strings.Split(string(output), "\n")
-
-	for i := 1; i < (len(outputLines) - 1); i++ {
-		require.Regexp(t, "^[0-9a-f]{44} refs/(heads|tags)/[^ ]+", outputLines[i])
+			require.Equal(t, "0000", outputLines[len(outputLines)-1])
+		})
 	}
-
-	require.Equal(t, "0000", outputLines[len(outputLines)-1])
 }
 
 func TestGitUploadArchiveSuccess(t *testing.T) {
 	ensureGitalyRepository(t)
 
 	client := runSSHD(t, successAPI(t))
-
-	session, err := client.NewSession()
-	require.NoError(t, err)
-	defer session.Close()
-
-	stdin, err := session.StdinPipe()
-	require.NoError(t, err)
-
-	stdout, err := session.StdoutPipe()
-	require.NoError(t, err)
-
+	session, stdin, stdout := newSession(t, client)
 	reader := bufio.NewReader(stdout)
 
-	err = session.Start(fmt.Sprintf("git-upload-archive %s", testRepo))
+	err := session.Start(fmt.Sprintf("git-upload-archive %s", testRepo))
 	require.NoError(t, err)
 
 	_, err = fmt.Fprintln(stdin, "0012argument HEAD\n0000")
+	require.NoError(t, err)
 
 	line, err := reader.ReadString('\n')
 	require.Equal(t, "0008ACK\n", line)
@@ -489,5 +582,23 @@ func TestGitUploadArchiveSuccess(t *testing.T) {
 	output, err := io.ReadAll(stdout)
 	require.NoError(t, err)
 
+	t.Logf("output: %q", output)
 	require.Equal(t, []byte("0000"), output[len(output)-4:])
 }
+
+func newSession(t *testing.T, client *ssh.Client) (*ssh.Session, io.WriteCloser, io.Reader) {
+	session, err := client.NewSession()
+	require.NoError(t, err)
+
+	stdin, err := session.StdinPipe()
+	require.NoError(t, err)
+
+	stdout, err := session.StdoutPipe()
+	require.NoError(t, err)
+
+	t.Cleanup(func() {
+		session.Close()
+	})
+
+	return session, stdin, stdout
+}
