@@ -9,7 +9,7 @@ import (
 	"sync"
 	"time"
 
-	yaml "gopkg.in/yaml.v2"
+	yaml "gopkg.in/yaml.v3"
 
 	"gitlab.com/gitlab-org/gitlab-shell/v14/client"
 	"gitlab.com/gitlab-org/gitlab-shell/v14/internal/gitaly"
@@ -23,6 +23,13 @@ const (
 
 type YamlDuration time.Duration
 
+type GSSAPIConfig struct {
+	Enabled              bool   `yaml:"enabled,omitempty"`
+	Keytab               string `yaml:"keytab,omitempty"`
+	ServicePrincipalName string `yaml:"service_principal_name,omitempty"`
+	LibPath              string
+}
+
 type ServerConfig struct {
 	Listen                  string       `yaml:"listen,omitempty"`
 	ProxyProtocol           bool         `yaml:"proxy_protocol,omitempty"`
@@ -41,6 +48,7 @@ type ServerConfig struct {
 	MACs                    []string     `yaml:"macs"`
 	KexAlgorithms           []string     `yaml:"kex_algorithms"`
 	Ciphers                 []string     `yaml:"ciphers"`
+	GSSAPI                  GSSAPIConfig `yaml:"gssapi,omitempty"`
 }
 
 type HttpSettingsConfig struct {
@@ -134,14 +142,8 @@ func (c *Config) HttpClient() (*client.HttpClient, error) {
 			return
 		}
 
-		if client.HTTPClient != nil {
-			tr := client.HTTPClient.Transport
-			client.HTTPClient.Transport = metrics.NewRoundTripper(tr)
-		}
-		if os.Getenv("FF_GITLAB_SHELL_RETRYABLE_HTTP") == "1" && client.RetryableHTTP != nil {
-			tr := client.RetryableHTTP.HTTPClient.Transport
-			client.RetryableHTTP.HTTPClient.Transport = metrics.NewRoundTripper(tr)
-		}
+		tr := client.RetryableHTTP.HTTPClient.Transport
+		client.RetryableHTTP.HTTPClient.Transport = metrics.NewRoundTripper(tr)
 
 		c.httpClient = client
 	})
