@@ -5,7 +5,7 @@ import (
 	"fmt"
 	"net/http"
 
-	pb "gitlab.com/gitlab-org/gitaly/v15/proto/go/gitalypb"
+	pb "gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
 	"gitlab.com/gitlab-org/gitlab-shell/v14/client"
 	"gitlab.com/gitlab-org/gitlab-shell/v14/internal/command/commandargs"
 	"gitlab.com/gitlab-org/gitlab-shell/v14/internal/config"
@@ -22,13 +22,17 @@ type Client struct {
 }
 
 type Request struct {
-	Action   commandargs.CommandType `json:"action"`
-	Repo     string                  `json:"project"`
-	Changes  string                  `json:"changes"`
-	Protocol string                  `json:"protocol"`
-	KeyId    string                  `json:"key_id,omitempty"`
-	Username string                  `json:"username,omitempty"`
-	CheckIp  string                  `json:"check_ip,omitempty"`
+	Action        commandargs.CommandType `json:"action"`
+	Repo          string                  `json:"project"`
+	Changes       string                  `json:"changes"`
+	Protocol      string                  `json:"protocol"`
+	KeyId         string                  `json:"key_id,omitempty"`
+	Username      string                  `json:"username,omitempty"`
+	Krb5Principal string                  `json:"krb5principal,omitempty"`
+	CheckIp       string                  `json:"check_ip,omitempty"`
+	// NamespacePath is the full path of the namespace in which the authenticated
+	// user is allowed to perform operation.
+	NamespacePath string `json:"namespace_path,omitempty"`
 }
 
 type Gitaly struct {
@@ -39,10 +43,13 @@ type Gitaly struct {
 }
 
 type CustomPayloadData struct {
-	ApiEndpoints []string `json:"api_endpoints"`
-	Username     string   `json:"gl_username"`
-	PrimaryRepo  string   `json:"primary_repo"`
-	UserId       string   `json:"gl_id,omitempty"`
+	ApiEndpoints                 []string          `json:"api_endpoints"`
+	Username                     string            `json:"gl_username"`
+	PrimaryRepo                  string            `json:"primary_repo"`
+	UserId                       string            `json:"gl_id,omitempty"`
+	RequestHeaders               map[string]string `json:"request_headers"`
+	GeoProxyDirectToPrimary      bool              `json:"geo_proxy_direct_to_primary"`
+	GeoProxyFetchDirectToPrimary bool              `json:"geo_proxy_fetch_direct_to_primary"`
 }
 
 type CustomPayload struct {
@@ -77,10 +84,18 @@ func NewClient(config *config.Config) (*Client, error) {
 }
 
 func (c *Client) Verify(ctx context.Context, args *commandargs.Shell, action commandargs.CommandType, repo string) (*Response, error) {
-	request := &Request{Action: action, Repo: repo, Protocol: protocol, Changes: anyChanges}
+	request := &Request{
+		Action:        action,
+		Repo:          repo,
+		Protocol:      protocol,
+		Changes:       anyChanges,
+		NamespacePath: args.Env.NamespacePath,
+	}
 
 	if args.GitlabUsername != "" {
 		request.Username = args.GitlabUsername
+	} else if args.GitlabKrb5Principal != "" {
+		request.Krb5Principal = args.GitlabKrb5Principal
 	} else {
 		request.KeyId = args.GitlabKeyId
 	}
