@@ -10,7 +10,7 @@ import (
 	"testing"
 
 	"github.com/stretchr/testify/require"
-	pb "gitlab.com/gitlab-org/gitaly/v15/proto/go/gitalypb"
+	pb "gitlab.com/gitlab-org/gitaly/v16/proto/go/gitalypb"
 	"gitlab.com/gitlab-org/gitlab-shell/v14/client/testserver"
 	"gitlab.com/gitlab-org/gitlab-shell/v14/internal/command/commandargs"
 	"gitlab.com/gitlab-org/gitlab-shell/v14/internal/config"
@@ -19,9 +19,11 @@ import (
 )
 
 var (
+	namespace         = "group"
 	repo              = "group/private"
 	receivePackAction = commandargs.ReceivePack
 	uploadPackAction  = commandargs.UploadPack
+	defaultEnv        = sshenv.Env{NamespacePath: namespace}
 )
 
 func buildExpectedResponse(who string) *Response {
@@ -56,7 +58,7 @@ func buildExpectedResponse(who string) *Response {
 func TestSuccessfulResponses(t *testing.T) {
 	okResponse := testResponse{body: responseBody(t, "allowed.json"), status: http.StatusOK}
 	client := setup(t,
-		map[string]testResponse{"first": okResponse},
+		map[string]testResponse{"first": okResponse, "test@TEST.TEST": okResponse},
 		map[string]testResponse{"1": okResponse},
 	)
 
@@ -71,7 +73,11 @@ func TestSuccessfulResponses(t *testing.T) {
 			who:  "key-1",
 		}, {
 			desc: "Provide username within the request",
-			args: &commandargs.Shell{GitlabUsername: "first"},
+			args: &commandargs.Shell{GitlabUsername: "first", Env: defaultEnv},
+			who:  "user-1",
+		}, {
+			desc: "Provide krb5principal within the request",
+			args: &commandargs.Shell{GitlabKrb5Principal: "test@TEST.TEST"},
 			who:  "user-1",
 		},
 	}
@@ -95,7 +101,7 @@ func TestGeoPushGetCustomAction(t *testing.T) {
 		},
 	}, nil)
 
-	args := &commandargs.Shell{GitlabUsername: "custom"}
+	args := &commandargs.Shell{GitlabUsername: "custom", Env: defaultEnv}
 	result, err := client.Verify(context.Background(), args, receivePackAction, repo)
 	require.NoError(t, err)
 
@@ -103,9 +109,11 @@ func TestGeoPushGetCustomAction(t *testing.T) {
 	response.Payload = CustomPayload{
 		Action: "geo_proxy_to_primary",
 		Data: CustomPayloadData{
-			ApiEndpoints: []string{"geo/proxy_git_ssh/info_refs_receive_pack", "geo/proxy_git_ssh/receive_pack"},
-			Username:     "custom",
-			PrimaryRepo:  "https://repo/path",
+			ApiEndpoints:            []string{"geo/proxy_git_ssh/info_refs_receive_pack", "geo/proxy_git_ssh/receive_pack"},
+			GeoProxyDirectToPrimary: true,
+			RequestHeaders:          map[string]string{"Authorization": "Bearer token"},
+			Username:                "custom",
+			PrimaryRepo:             "https://repo/path",
 		},
 	}
 	response.StatusCode = 300
@@ -122,7 +130,7 @@ func TestGeoPullGetCustomAction(t *testing.T) {
 		},
 	}, nil)
 
-	args := &commandargs.Shell{GitlabUsername: "custom"}
+	args := &commandargs.Shell{GitlabUsername: "custom", Env: defaultEnv}
 	result, err := client.Verify(context.Background(), args, uploadPackAction, repo)
 	require.NoError(t, err)
 
@@ -130,9 +138,11 @@ func TestGeoPullGetCustomAction(t *testing.T) {
 	response.Payload = CustomPayload{
 		Action: "geo_proxy_to_primary",
 		Data: CustomPayloadData{
-			ApiEndpoints: []string{"geo/proxy_git_ssh/info_refs_upload_pack", "geo/proxy_git_ssh/upload_pack"},
-			Username:     "custom",
-			PrimaryRepo:  "https://repo/path",
+			ApiEndpoints:                 []string{"geo/proxy_git_ssh/info_refs_upload_pack", "geo/proxy_git_ssh/upload_pack"},
+			Username:                     "custom",
+			GeoProxyFetchDirectToPrimary: true,
+			PrimaryRepo:                  "https://repo/path",
+			RequestHeaders:               map[string]string{"Authorization": "Bearer token"},
 		},
 	}
 	response.StatusCode = 300
@@ -255,6 +265,11 @@ func setup(t *testing.T, userResponses, keyResponses map[string]testResponse) *C
 					w.WriteHeader(tr.status)
 					_, err := w.Write(tr.body)
 					require.NoError(t, err)
+					require.Equal(t, namespace, requestBody.NamespacePath)
+				} else if tr, ok := userResponses[requestBody.Krb5Principal]; ok {
+					w.WriteHeader(tr.status)
+					_, err := w.Write(tr.body)
+					require.NoError(t, err)
 				} else if tr, ok := keyResponses[requestBody.KeyId]; ok {
 					w.WriteHeader(tr.status)
 					_, err := w.Write(tr.body)
